@@ -31,8 +31,7 @@
 
 #define DRANAP _ranap_DRANAP
 
-static int cn_ranap_rx_initiating_msg_co(void *ctx, RANAP_InitiatingMessage_t *imsg,
-					 ranap_message *message)
+static int cn_ranap_rx_initiating_msg_co(RANAP_InitiatingMessage_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -74,8 +73,8 @@ static int cn_ranap_rx_initiating_msg_co(void *ctx, RANAP_InitiatingMessage_t *i
 		rc = ranap_decode_iu_releasecommandies(&message->msg.iu_ReleaseCommandIEs, &imsg->value);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "Procedure %s (CO, IM) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Received RANAP Procedure %s (CO, IM) from RNC. Decode not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, imsg->procedureCode));
 		rc = -1;
 		break;
@@ -118,15 +117,14 @@ static void cn_ranap_free_initiating_msg_co(ranap_message *message)
 		ranap_free_iu_releasecommandies(&message->msg.iu_ReleaseCommandIEs);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Not freeing suspicious RANAP "
-		     "Procedure %s (CO, IM) from RNC\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP Procedure %s (CO, IM) from RNC not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, message->procedureCode));
 		break;
 	}
 }
 
-static int cn_ranap_rx_successful_msg_co(void *ctx, RANAP_SuccessfulOutcome_t *imsg,
-					 ranap_message *message)
+static int cn_ranap_rx_successful_msg_co(RANAP_SuccessfulOutcome_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -150,8 +148,8 @@ static int cn_ranap_rx_successful_msg_co(void *ctx, RANAP_SuccessfulOutcome_t *i
 		rc = ranap_decode_iu_releasecompleteies(&message->msg.iu_ReleaseCompleteIEs, &imsg->value);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "Procedure %s (SO) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Received RANAP Procedure %s (SO) from RNC. Decode not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, imsg->procedureCode));
 		rc = -1;
 		break;
@@ -176,15 +174,14 @@ static void cn_ranap_free_successful_msg_co(ranap_message *message)
 		ranap_free_iu_releasecompleteies(&message->msg.iu_ReleaseCompleteIEs);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Not freeing suspicious RANAP "
-		     "Procedure %s (SO) from RNC\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP Procedure %s (SO) from RNC not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, message->procedureCode));
 		break;
 	}
 }
 
-static int cn_ranap_rx_outcome_msg_co(void *ctx, RANAP_Outcome_t *imsg,
-					ranap_message *message)
+static int cn_ranap_rx_outcome_msg_co(RANAP_Outcome_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -200,8 +197,8 @@ static int cn_ranap_rx_outcome_msg_co(void *ctx, RANAP_Outcome_t *imsg,
 		rc = ranap_decode_rab_assignmentresponseies(&message->msg.raB_AssignmentResponseIEs, &imsg->value);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "Procedure %s (O) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Received RANAP Procedure %s (O) from RNC. Decode not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, imsg->procedureCode));
 		rc = -1;
 		break;
@@ -218,23 +215,23 @@ static void cn_ranap_free_outcome_msg_co(ranap_message *message)
 		ranap_free_rab_assignmentresponseies(&message->msg.raB_AssignmentResponseIEs);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Not freeing suspicious RANAP "
-		     "Procedure %s (O) from RNC\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP Procedure %s (O) from RNC not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, message->procedureCode));
 		break;
 	}
 }
 
-static int _cn_ranap_rx_co(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *message)
+static int _cn_ranap_rx_co(RANAP_RANAP_PDU_t *pdu, ranap_message *message)
 {
 	int rc = 0;
 
 	switch (pdu->present) {
 	case RANAP_RANAP_PDU_PR_initiatingMessage:
-		rc = cn_ranap_rx_initiating_msg_co(ctx, &pdu->choice.initiatingMessage, message);
+		rc = cn_ranap_rx_initiating_msg_co(&pdu->choice.initiatingMessage, message);
 		break;
 	case RANAP_RANAP_PDU_PR_successfulOutcome:
-		rc = cn_ranap_rx_successful_msg_co(ctx, &pdu->choice.successfulOutcome, message);
+		rc = cn_ranap_rx_successful_msg_co(&pdu->choice.successfulOutcome, message);
 		break;
 	case RANAP_RANAP_PDU_PR_unsuccessfulOutcome:
 		LOGP(DRANAP, LOGL_NOTICE, "Received unsupported RANAP "
@@ -244,11 +241,11 @@ static int _cn_ranap_rx_co(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *mes
 		rc = -1;
 		break;
 	case RANAP_RANAP_PDU_PR_outcome:
-		rc = cn_ranap_rx_outcome_msg_co(ctx, &pdu->choice.outcome, message);
+		rc = cn_ranap_rx_outcome_msg_co(&pdu->choice.outcome, message);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "presence %s (CO) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Received RANAP Procedure %s (CO) from RNC. Decode not implemented\n",
 		     get_value_string(ranap_presence_vals, pdu->present));
 		rc = -1;
 		break;
@@ -257,7 +254,8 @@ static int _cn_ranap_rx_co(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *mes
 	return rc;
 }
 
-static void _cn_ranap_free_co(ranap_message *message)
+/* free a decoded connection-oriented RANAP message */
+void ranap_cn_rx_co_free(ranap_message *message)
 {
 	switch (message->direction) {
 	case RANAP_RANAP_PDU_PR_initiatingMessage:
@@ -267,57 +265,70 @@ static void _cn_ranap_free_co(ranap_message *message)
 		cn_ranap_free_successful_msg_co(message);
 		break;
 	case RANAP_RANAP_PDU_PR_unsuccessfulOutcome:
-		LOGP(DRANAP, LOGL_NOTICE, "Not freeing unsupported RANAP "
-		     "unsuccessful outcome procedure (CO) from RNC\n");
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP unsuccessful outcome procedure (CO) from RNC not implemented\n");
 		break;
 	case RANAP_RANAP_PDU_PR_outcome:
 		cn_ranap_free_outcome_msg_co(message);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "presence %s (CO) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP Procedure %s (CO) from RNC not implemented\n",
 		     get_value_string(ranap_presence_vals, message->direction));
 		break;
 	}
 }
 
-/* receive a connection-oriented RANAP message and call
- * cn_ranap_handle_co() with the resulting ranap_message struct */
-int ranap_cn_rx_co(ranap_handle_cb cb, void *ctx, uint8_t *data, size_t len)
+/* decode a connection-oriented RANAP message */
+int ranap_cn_rx_co_decode2(ranap_message *message, uint8_t *data, size_t len)
 {
 	RANAP_RANAP_PDU_t *pdu = NULL;
-	ranap_message message;
 	asn_dec_rval_t dec_ret;
 	int rc;
 
-	memset(&message, 0, sizeof(message));
+	memset(message, 0, sizeof(*message));
 
-	dec_ret = aper_decode(NULL,&asn_DEF_RANAP_RANAP_PDU, (void **) &pdu,
-			      data, len, 0, 0);
+	dec_ret = aper_decode(NULL, &asn_DEF_RANAP_RANAP_PDU, (void **)&pdu, data, len, 0, 0);
 	if (dec_ret.code != RC_OK) {
 		LOGP(DRANAP, LOGL_ERROR, "Error in RANAP ASN.1 decode\n");
 		return -1;
 	}
 
-	message.direction = pdu->present;
+	message->direction = pdu->present;
 
-	rc = _cn_ranap_rx_co(ctx, pdu, &message);
-
-	if (rc == 0)
-		(*cb)(ctx, &message);
-	else
-		LOGP(DRANAP, LOGL_ERROR, "Not calling cn_ranap_handle_co() due to rc=%d\n", rc);
-
-	/* Free the asn1 structs in message */
-	_cn_ranap_free_co(&message);
+	rc = _cn_ranap_rx_co(pdu, message);
 
 	ASN_STRUCT_FREE(asn_DEF_RANAP_RANAP_PDU, pdu);
 
 	return rc;
 }
 
-static int cn_ranap_rx_initiating_msg_cl(void *ctx, RANAP_InitiatingMessage_t *imsg,
-					 ranap_message *message)
+int ranap_cn_rx_co_decode(void *unused, ranap_message *message, uint8_t *data, size_t len)
+{
+	return ranap_cn_rx_co_decode2(message, data, len);
+}
+
+/* receive a connection-oriented RANAP message and call
+ * cn_ranap_handle_co() with the resulting ranap_message struct */
+int ranap_cn_rx_co(ranap_handle_cb cb, void *priv, uint8_t *data, size_t len)
+{
+	ranap_message message;
+	int rc;
+
+	rc = ranap_cn_rx_co_decode2(&message, data, len);
+
+	if (rc == 0)
+		(*cb)(priv, &message);
+	else
+		LOGP(DRANAP, LOGL_ERROR, "Not calling cn_ranap_handle_co() due to rc=%d\n", rc);
+
+	/* Free the asn1 structs in message */
+	ranap_cn_rx_co_free(&message);
+
+	return rc;
+}
+
+static int cn_ranap_rx_initiating_msg_cl(RANAP_InitiatingMessage_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -355,8 +366,8 @@ static int cn_ranap_rx_initiating_msg_cl(void *ctx, RANAP_InitiatingMessage_t *i
 		rc = ranap_decode_pagingies(&message->msg.pagingIEs, &imsg->value);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "Procedure %s (CL, IM) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Received RANAP Procedure %s (CL, IM) from RNC. Decode not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, imsg->procedureCode));
 		break;
 	}
@@ -395,15 +406,14 @@ static void cn_ranap_free_initiating_msg_cl(ranap_message *message)
 		ranap_free_pagingies(&message->msg.pagingIEs);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Not freeing suspicious RANAP "
-		     "Procedure %s (CL, IM)\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP Procedure %s (CL, IM) not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, message->procedureCode));
 		break;
 	}
 }
 
-static int cn_ranap_rx_successful_msg_cl(void *ctx, RANAP_SuccessfulOutcome_t *imsg,
-					 ranap_message *message)
+static int cn_ranap_rx_successful_msg_cl(RANAP_SuccessfulOutcome_t *imsg, ranap_message *message)
 {
 	int rc = 0;
 
@@ -430,8 +440,8 @@ static int cn_ranap_rx_successful_msg_cl(void *ctx, RANAP_SuccessfulOutcome_t *i
 		rc = ranap_decode_uplinkinformationexchangeresponseies(&message->msg.uplinkInformationExchangeResponseIEs, &imsg->value);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "Procedure %s (CL, SO) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Received RANAP Procedure %s (CL, SO) from RNC. Decode not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, imsg->procedureCode));
 		break;
 	}
@@ -458,14 +468,14 @@ static void cn_ranap_free_successful_msg_cl(ranap_message *message)
 		ranap_free_uplinkinformationexchangeresponseies(&message->msg.uplinkInformationExchangeResponseIEs);
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Not freeing suspicious RANAP "
-		     "Procedure %s (CL, SO)\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP Procedure %s (CL, SO) from RNC not implemented\n",
 		     get_value_string(ranap_procedure_code_vals, message->procedureCode));
 		break;
 	}
 }
 
-static int _cn_ranap_rx_cl(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *message)
+static int _cn_ranap_rx_cl(RANAP_RANAP_PDU_t *pdu, ranap_message *message)
 {
 	int rc = 0;
 
@@ -473,12 +483,10 @@ static int _cn_ranap_rx_cl(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *mes
 
 	switch (pdu->present) {
 	case RANAP_RANAP_PDU_PR_initiatingMessage:
-		rc = cn_ranap_rx_initiating_msg_cl(ctx, &pdu->choice.initiatingMessage,
-						   message);
+		rc = cn_ranap_rx_initiating_msg_cl(&pdu->choice.initiatingMessage, message);
 		break;
 	case RANAP_RANAP_PDU_PR_successfulOutcome:
-		rc = cn_ranap_rx_successful_msg_cl(ctx, &pdu->choice.successfulOutcome,
-						   message);
+		rc = cn_ranap_rx_successful_msg_cl(&pdu->choice.successfulOutcome, message);
 		break;
 	case RANAP_RANAP_PDU_PR_unsuccessfulOutcome:
 		LOGP(DRANAP, LOGL_NOTICE, "Received unsupported RANAP "
@@ -487,8 +495,8 @@ static int _cn_ranap_rx_cl(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *mes
 			     	      pdu->choice.unsuccessfulOutcome.procedureCode));
 		break;
 	default:
-		LOGP(DRANAP, LOGL_NOTICE, "Received suspicious RANAP "
-		     "presence %s (CL) from RNC, ignoring\n",
+		LOGP(DRANAP, LOGL_INFO,
+		     "Received RANAP Procedure %s (CL) from RNC. Decode not implemented\n",
 		     get_value_string(ranap_presence_vals, pdu->present));
 		break;
 	}
@@ -496,7 +504,8 @@ static int _cn_ranap_rx_cl(void *ctx, RANAP_RANAP_PDU_t *pdu, ranap_message *mes
 	return rc;
 }
 
-static void _cn_ranap_free_cl(ranap_message *message)
+/* free a decoded connection-less RANAP message */
+void ranap_cn_rx_cl_free(ranap_message *message)
 {
 	switch (message->direction) {
 	case RANAP_RANAP_PDU_PR_initiatingMessage:
@@ -506,8 +515,8 @@ static void _cn_ranap_free_cl(ranap_message *message)
 		cn_ranap_free_successful_msg_cl(message);
 		break;
 	case RANAP_RANAP_PDU_PR_unsuccessfulOutcome:
-		LOGP(DRANAP, LOGL_NOTICE, "Not freeing unsupported RANAP "
-		     "unsuccessful outcome procedure from RNC\n");
+		LOGP(DRANAP, LOGL_INFO,
+		     "Freeing RANAP unsuccessful outcome procedure from RNC not implemented\n");
 		break;
 	default:
 		LOGP(DRANAP, LOGL_NOTICE, "Suspicious RANAP "
@@ -516,37 +525,51 @@ static void _cn_ranap_free_cl(ranap_message *message)
 	}
 }
 
-/* receive a connection-less RANAP message and call
- * cn_ranap_handle_co() with the resulting ranap_message struct */
-int ranap_cn_rx_cl(ranap_handle_cb cb, void *ctx, uint8_t *data, size_t len)
+/* decode a connection-less RANAP message */
+int ranap_cn_rx_cl_decode2(ranap_message *message, uint8_t *data, size_t len)
 {
 	RANAP_RANAP_PDU_t *pdu = NULL;
-	ranap_message message;
 	asn_dec_rval_t dec_ret;
 	int rc;
 
-	memset(&message, 0, sizeof(message));
+	memset(message, 0, sizeof(*message));
 
-	dec_ret = aper_decode(NULL,&asn_DEF_RANAP_RANAP_PDU, (void **) &pdu,
-			      data, len, 0, 0);
+	dec_ret = aper_decode(NULL, &asn_DEF_RANAP_RANAP_PDU, (void **)&pdu, data, len, 0, 0);
 	if (dec_ret.code != RC_OK) {
 		LOGP(DRANAP, LOGL_ERROR, "Error in RANAP ASN.1 decode\n");
 		return -1;
 	}
 
-	message.direction = pdu->present;
+	message->direction = pdu->present;
 
-	rc = _cn_ranap_rx_cl(ctx, pdu, &message);
-
-	if (rc == 0)
-		(*cb)(ctx, &message);
-	else
-		LOGP(DRANAP, LOGL_ERROR, "Not calling cn_ranap_handle_cl() due to rc=%d\n", rc);
-
-	/* Free the asn1 structs in message */
-	_cn_ranap_free_cl(&message);
+	rc = _cn_ranap_rx_cl(pdu, message);
 
 	ASN_STRUCT_FREE(asn_DEF_RANAP_RANAP_PDU, pdu);
 
 	return rc;
 }
+
+int ranap_cn_rx_cl_decode(void *unused, ranap_message *message, uint8_t *data, size_t len)
+{
+	return ranap_cn_rx_cl_decode2(message, data, len);
+}
+
+/* receive a connection-less RANAP message and call
+ * cn_ranap_handle_co() with the resulting ranap_message struct */
+int ranap_cn_rx_cl(ranap_handle_cb cb, void *priv, uint8_t *data, size_t len)
+{
+	ranap_message message;
+	int rc;
+
+	rc = ranap_cn_rx_cl_decode2(&message, data, len);
+
+	if (rc == 0)
+		(*cb)(priv, &message);
+	else
+		LOGP(DRANAP, LOGL_ERROR, "Not calling cn_ranap_handle_cl() due to rc=%d\n", rc);
+
+	/* Free the asn1 structs in message */
+	ranap_cn_rx_cl_free(&message);
+
+	return rc;
+}
